/** @file fwUnitTestComponentRemoveComments.ctl

@par Creation Date
		01/04/2014    
  
*/
const char SPACE_CHAR = 32;
const char DOUBLEQUOTE_CHAR = 34;
const char SINGLEQUOTE_CHAR = 39;
const char SLASH_CHAR = 47;
const char QUESTION_CHAR = 63;
const char BACKSLASH_CHAR = 92 ;

/** 
  
  @par Usage
	  Public

  @param[in]  fileString dyn_string
  @param[out] fileStringNoComment dyn_string
  
*/
int fwUnitTestComponentRemoveComments_RemoveComments(dyn_string fileString, dyn_string &fileStringNoComment)
{
  bool commentMode = FALSE;
  bool commentMultiLine = FALSE; // traces comments started by /* (determines if we are still in comment mode at the beginning of a new line)

  bool stringMode = FALSE;
  string stringStarted; // DOUBLEQUOTE or QUOTE
  // a string started by " should also end by ", and a string started by ' should also end by '  
  
  string buf; // the current line of code
  string newbuf; // the current line of code without any comments
  
  int consecutive_backslash; // this variable keeps the count of how many consecutive \ characters have been
                             // accumulated; when finding a " or a ' char, its even/odd parity will be checked
								          // this is because " means something, but \" means something else. And \\" is the same as "
								          // because the \\ are eliminated. But \\\" is the same as \", and so on!
  dynClear(fileStringNoComment);
  
  //loop over the lines of the script
  for(int i = 1; i <= dynlen(fileString); i++) 
  {  
    buf = fileString[i];    
    newbuf = "";
    commentMode = commentMultiLine;  
	 
    // loops through each char in buf. 
    // in sequences with even number of \, like \\", \\\\". etc, the " is a string terminator
    // in sequences with odd number of \, like \", \\\". etc, the " is a character in the string
    consecutive_backslash = 0;     
    for(int j = 0; j < strlen(buf); j++) 
    {
      if (!commentMode)   // we only interpret thing if we are not in Comment Mode 
      {                     // (apart for searching for */, which is done in the else of this if)
        
        // parse " section
        if (buf[j] == DOUBLEQUOTE_CHAR) 
        {   
          if ((consecutive_backslash % 2) == 1)  // found \", or \\\", etc.
          { 
            if (stringMode) 
            {                
              newbuf = newbuf + buf[j];    
              consecutive_backslash = 0;  
              continue; // continue looping over characters in buf.                   
            }                                    
          }                                       
          if (stringMode) 
          { // if we are here, consecutive_backslash was even (otherwise the above "continue" would have been executed)
            if (stringStarted == "DOUBLEQUOTE") 
            {
              stringMode = FALSE;
              stringStarted = "";
            }
          } 
          else  // found ", or \\", etc.
          {  //TRICK: if we were not in string mode, the only possibility here is actually " (without any \); otherwise we 
		                 // would have had a PVSS syntax error!
            stringMode = TRUE;
            stringStarted = "DOUBLEQUOTE";
          }
        }
        
        // parse ' section. This is done in the same way as DOUBLEQUOTE_CHAR
        if (buf[j] == SINGLEQUOTE_CHAR) 
        {     
          if ((consecutive_backslash % 2) == 1) // found \", or \\\", etc. 
          { 
            if (stringMode) 
            {               
               newbuf = newbuf+buf[j];    
               consecutive_backslash = 0;  
               continue; // continue looping over characters in buf.                  
            }                                    
          }                                        
          if (stringMode) 
          { // if we are here, consecutive_backslash was even (otherwise the above "continue" would have been executed)
            if (stringStarted == "QUOTE") 
            {
              stringMode = FALSE;
              stringStarted = "";
            }
          } 
          else// found ", or \\", etc.
          { //TRICK: if we were not in string mode, the only possibility here is actually " (without any \); otherwise we 
		                // would have had a PVSS syntax error!
            stringMode = TRUE;
            stringStarted = "QUOTE";
          }
        }        

        if (buf[j] == BACKSLASH_CHAR) 
        {
          ++consecutive_backslash; 
        }      
        else 
        {
          consecutive_backslash = 0;
        }          

        // now detecting beginning or end of comments
        if ((buf[j] != '/') || (stringMode)) // if character is not / (or if we are in string mode) add character to newbuf
        {
          newbuf = newbuf+buf[j];
          continue;
        }
        // from here onwards, we are not in StringMode
        if ((buf[j] == '/') && (buf[j+1] == '/')) // found line comment start
        {
          commentMode = TRUE;
          break;  // if it finds the // pattern, the rest of the line is a comment and will be skipped
        }
        
        if ((buf[j] == '/') && (buf[j+1] == '*')) // found multiline comment start
        {
          commentMultiLine = TRUE;
          commentMode = TRUE;
          continue;
        }
        if (buf[j] == '/') // do not forget to add to the buffer a single isolated /
        {                  // eg. the SLASH is used for a division
          newbuf = newbuf + buf[j];
          continue;
        }  
      } 
      else { // here we were already in comment mode 
	            // we check for the sequence */, which would close a comment started by /*
        if ((buf[j] == '*') && (buf[j+1] == '/') && commentMultiLine) 
        {         
          commentMultiLine = FALSE;
          commentMode = FALSE;
          j=j+1;
        }
      }
    }
    
    if (strlen(newbuf)==0) 
    {
      continue;
    }
    if (newbuf[strlen(newbuf)-1] != '\n') 
    {
      newbuf = newbuf + '\n';
    }       
    dynAppend(fileStringNoComment, newbuf);
  }
  return(0);
}
