/** @file fwUnitTestComponentUpdateGui.ctl 

@brief Allows the updating of the panel's shapes as the tests are running. 
          This is so the user is given instant feedback.

@par Creation Date
		01/04/2014          
          
*/

//----------------------------------------------------------------------------------------------

/** 
  @par Description: Displays the results of the tests in the tree.
  
  @par Usage
	  public

  @param[in]  textId string, The string identifying the branch in the tree.
  @param[in]  success bool
  @param[in]  failedMessage string
  @param[in]  testSelectionTree shape
*/
void fwUnitTestComponentUpdateGui_updateGuiTree(string textId, bool success, string failedMessage, string strExecTime, shape testSelectionTree)
{
  if (testSelectionTree != 0) // check if we are using the panel
  {
    string testResultText;
    string testResultColour;
    if (success)
    {
      testResultText = "Passed";
      testResultColour = "{128,255,128}";//Green
    }
    else
    {
      testResultText = "Failed";
      testResultColour = "{255,128,128}";//Red
    }
    
    // update the GUI objects 
    // the magic numbers are only used here. See panel scopeLib displayTest() for where the columns are created.
    testSelectionTree.setBackColor(textId, 0, testResultColour);
    testSelectionTree.setBackColor(textId, 1, testResultColour);
    testSelectionTree.setBackColor(textId, 2, testResultColour);
    testSelectionTree.setBackColor(textId, 3, testResultColour);
    testSelectionTree.setText(textId, 1, testResultText);
    testSelectionTree.setText(textId, 2, failedMessage);
    
    if (strExecTime == "0") 
    {
      strExecTime = "<1";  
    }      
    testSelectionTree.setText(textId, 3, strExecTime);
  }
}

//----------------------------------------------------------------------------------------------

/** 
  @par Description: Updates the progress bar and the feedback text.
  
  @par Usage
	  public

  @param[in]  testsPassed float 
  @param[in]  testsCompleted float 
  @param[in]  testProgressBar shape
  @param[in]  feedbackText shape
  @param[in]  feedbackProgressText shape
  @param[in]  numTestsToRun int
*/   
void fwUnitTestComponentUpdateGui_displayProgress(float testsPassed, float testsCompleted, shape testProgressBar, shape feedbackText, shape feedbackProgressText, int numTestsToRun)
{
  if (testProgressBar != 0)
  {
    //feedbackProgressText.visible(true); // is now always visible.    
    feedbackProgressText.text(testsCompleted + "/" + numTestsToRun);
    testProgressBar.progress((testsCompleted / numTestsToRun) * 100);
                                                                    // If you change the "Running tests" here also update the panel scopeLib.startTests() "strreplace(textFromFeedbackText, "Running tests", "Tests finished");" line.
    feedbackText.text("" + testsPassed + "/" + testsCompleted + " (" + strformat("\\left{%6.2}", 6, (testsPassed / testsCompleted) * 100) + "%) tests passed.  " + "Running tests");
    //feedbackText.text("" + testsPassed + "/" + testsCompleted + " (" + (testsPassed / testsCompleted) * 100 + "%) tests passed.  " + "Running tests");
        
    //feedbackText.visible(true);
  }
}

//----------------------------------------------------------------------------------------------
