/**@file unitTestComponentAsserts.ctl  
  
@brief This file contains the assert functions.
  
@par Creation Date
		01/04/2014   
  
 */

#uses "CtrlPv2Admin"

/** 
  
  @par Usage : Public

  @param[in] expectedMessage string  
  @param[in] exceptionInfo dyn_string  
  @param[in] errorMsg string  
*/
void assertError(string expectedSeverity, string expectedMessage, dyn_string &exceptionInfo)
{
    string actualSeverity = exceptionInfo[1];
    string actualMessage = exceptionInfo[2];
    string actualDetails = exceptionInfo[3];
    
    if   ( ( expectedSeverity != actualSeverity )
                        ||
          ( expectedMessage != actualMessage )
         )
       { DebugTN("fwUnitTestComponentAsserts.ctl :  Entering assertError with");
         DebugTN("expectedSeverity = ", expectedSeverity);
         DebugTN("actualSeverity = ", actualSeverity);
         DebugTN("expectedMessage = " , expectedMessage);
         DebugTN("actualMessage = ", actualMessage);
         DebugTN("actualDetails = ",actualDetails);
           // /////////////////////////if (exceptionInfo != makeDynString("ERROR", expectedMessage, "")) {
           throw( makeError( "", PRIO_SEVERE, ERR_IMPL, 86, "Expected error message: " + expectedMessage + ", actual exceptionInfo: " + exceptionInfo ) );
       }
    dynClear(exceptionInfo);
}

//----------------------------------------------------------------------------------------------------------

/** 
  
  @par Usage: Public

  @param[in] actual dyn_mixed  
  @param[in] errorMsg string  
*/
void assertEmpty(dyn_mixed actual, string errorMsg = "" )
{
    if (dynlen(actual) != 0) {
        throw( makeError( "", PRIO_SEVERE, ERR_IMPL, 86, "Array is not empty: " + actual, errorMsg ) );
    }
}

//----------------------------------------------------------------------------------------------------------

/** 
  
  @par Usage: Public

  @param[in] expected mixed           
  @param[in] actual mixed  
  @param[in] errorMsg string  
*/
void assertEqual( mixed expected, mixed actual, string errorMsg = "" )
{
  bool same = (expected == actual);
  
  if( ! same )
  {
    errClass error = makeError( "", PRIO_SEVERE, ERR_IMPL, 86, "Expected: " + expected + ", actual: " + actual, errorMsg );
    throw( error );
  }
}

//----------------------------------------------------------------------------------------------------------

/** 
  
  @par Usage: Public

  @param[in] assertion bool           
  @param[in] errorMsg string  
*/
void assert( bool assertion, string errorMsg = "" )
{
  if( assertion == false )
  {
    if( errorMsg == "" ) errorMsg = "Assertion fail";
    errClass error = makeError( "", PRIO_SEVERE, ERR_IMPL, 86, errorMsg );
    throw( error );
  }
}

//----------------------------------------------------------------------------------------------------------

/** 
  
  @par Usage: Public

  @param[in] assertion bool           
  @param[in] errorMsg string  
*/
void assertTrue( bool assertion, string errorMsg = "" )
{
  if( assertion == false )
  {
    if( errorMsg == "" ) errorMsg = "Assertion fail";
    errClass error = makeError( "", PRIO_SEVERE, ERR_IMPL, 86, errorMsg );
    throw( error );
  }
}

//----------------------------------------------------------------------------------------------------------

/** 
  
  @par Usage: Public

  @param[in] assertion bool           
  @param[in] errorMsg string  
*/
void assertFalse( bool assertion, string errorMsg = "" )
{
  if( assertion == true )
  {
    if( errorMsg == "" ) errorMsg = "Assertion fail";
    errClass error = makeError( "", PRIO_SEVERE, ERR_IMPL, 86, errorMsg );
    throw( error );
  }
}
